@@ -123,10 +123,20 @@ void ConvertVkRenderPassCreateInfoToV2KHR(const VkRenderPassCreateInfo& in_struc
     using std::vector;
     const auto multiview_info = lvl_find_in_chain<VkRenderPassMultiviewCreateInfo>(in_struct.pNext);
     const auto* input_attachment_aspect_info = lvl_find_in_chain<VkRenderPassInputAttachmentAspectCreateInfo>(in_struct.pNext);
+    const auto fragment_density_map_info = lvl_find_in_chain<VkRenderPassFragmentDensityMapCreateInfoEXT>(in_struct.pNext);
 
     out_struct->~safe_VkRenderPassCreateInfo2();
     out_struct->sType = VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO_2_KHR;
-    out_struct->pNext = nullptr;
+
+    // Fixup RPCI2 pNext chain.  Only FDM2 is valid on both chains.
+    if (fragment_density_map_info) {
+        out_struct->pNext = SafePnextCopy(fragment_density_map_info);
+        auto base_struct = reinterpret_cast<const VkBaseOutStructure*>(out_struct->pNext);
+        const_cast<VkBaseOutStructure*>(base_struct)->pNext = nullptr;
+    } else {
+        out_struct->pNext = nullptr;
+    }
+
     out_struct->flags = in_struct.flags;
     out_struct->attachmentCount = in_struct.attachmentCount;
     out_struct->pAttachments = nullptr;  // to be filled
