@@ -255,10 +255,15 @@ void UtilPreCallRecordCreateDevice(VkPhysicalDevice gpu, safe_VkDeviceCreateInfo
     }
     if (features) {
         VkBool32 *desired = reinterpret_cast<VkBool32 *>(&desired_features);
-        VkBool32 *featurePtr = reinterpret_cast<VkBool32 *>(&features);
+        VkBool32 *featurePtr = reinterpret_cast<VkBool32 *>(features);
         VkBool32 *supported = reinterpret_cast<VkBool32 *>(&supported_features);
         for (size_t i = 0; i < sizeof(VkPhysicalDeviceFeatures); i += (sizeof(VkBool32))) {
-            *featurePtr++ |= (*supported++ & *desired++);
+            if (*supported && *desired) {
+                *featurePtr = true;
+            }
+            supported++;
+            desired++;
+            featurePtr++;
         }
     } else {
         VkPhysicalDeviceFeatures new_features = {};
@@ -323,6 +328,14 @@ void UtilGenerateStageMessage(const uint32_t *debug_record, std::string &msg) {
             strm << "Stage = Callable.  Global Launch ID (x,y,z) = (" << debug_record[kInstRayTracingOutLaunchIdX] << ", "
                  << debug_record[kInstRayTracingOutLaunchIdY] << ", " << debug_record[kInstRayTracingOutLaunchIdZ] << "). ";
         } break;
+        case spv::ExecutionModelTaskNV: {
+            strm << "Stage = Task. Global invocation ID (x, y, z) = (" << debug_record[kInstTaskOutGlobalInvocationIdX] << ", "
+                 << debug_record[kInstTaskOutGlobalInvocationIdY] << ", " << debug_record[kInstTaskOutGlobalInvocationIdZ] << " )";
+        } break;
+        case spv::ExecutionModelMeshNV: {
+            strm << "Stage = Mesh.Global invocation ID (x, y, z) = (" << debug_record[kInstMeshOutGlobalInvocationIdX] << ", "
+                 << debug_record[kInstMeshOutGlobalInvocationIdY] << ", " << debug_record[kInstMeshOutGlobalInvocationIdZ] << " )";
+        } break;
         default: {
             strm << "Internal Error (unexpected stage = " << debug_record[kInstCommonOutStageIdx] << "). ";
             assert(false);
@@ -436,7 +449,7 @@ bool GetLineAndFilename(const std::string string, uint32_t *linenumber, std::str
         }
     }
     if (0 == line_index) return false;
-    *linenumber = std::stoul(tokens[line_index]);
+    *linenumber = static_cast<uint32_t>(std::stoul(tokens[line_index]));
     uint32_t filename_index = line_index + 1;
     // Remove enclosing double quotes around filename
     if (size > filename_index) filename = tokens[filename_index].substr(1, tokens[filename_index].size() - 2);
@@ -528,7 +541,10 @@ void UtilGenerateSourceMessages(const std::vector<unsigned int> &pgm, const uint
             }
         }
         if (!found_opstring) {
-            filename_stream << "Unable to find SPIR-V OpString for file id " << reported_file_id << " from OpLine instruction.";
+            filename_stream << "Unable to find SPIR-V OpString for file id " << reported_file_id << " from OpLine instruction."
+                            << std::endl;
+            filename_stream << "File ID = " << reported_file_id << ", Line Number = " << reported_line_number
+                            << ", Column = " << reported_column_number << std::endl;
         }
     }
     filename_msg = filename_stream.str();
