@@ -67,6 +67,7 @@ class ParameterValidationGeneratorOptions(GeneratorOptions):
                  conventions = None,
                  filename = None,
                  directory = '.',
+                 genpath = None,
                  apiname = None,
                  profile = None,
                  versions = '.*',
@@ -85,9 +86,20 @@ class ParameterValidationGeneratorOptions(GeneratorOptions):
                  alignFuncParam = 0,
                  expandEnumerants = True,
                  valid_usage_path = ''):
-        GeneratorOptions.__init__(self, conventions, filename, directory, apiname, profile,
-                                  versions, emitversions, defaultExtensions,
-                                  addExtensions, removeExtensions, emitExtensions, sortProcedure)
+        GeneratorOptions.__init__(self,
+                conventions = conventions,
+                filename = filename,
+                directory = directory,
+                genpath = genpath,
+                apiname = apiname,
+                profile = profile,
+                versions = versions,
+                emitversions = emitversions,
+                defaultExtensions = defaultExtensions,
+                addExtensions = addExtensions,
+                removeExtensions = removeExtensions,
+                emitExtensions = emitExtensions,
+                sortProcedure = sortProcedure)
         self.prefixText      = prefixText
         self.apicall         = apicall
         self.apientry        = apientry
@@ -126,6 +138,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
         self.INDENT_SPACES = 4
         self.declarations = []
 
+
         inline_custom_source_preamble = """
 """
 
@@ -137,6 +150,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             'vkCreateDevice',
             'vkCreateQueryPool',
             'vkCreateRenderPass',
+            'vkCreateRenderPass2',
             'vkCreateRenderPass2KHR',
             'vkCreateBuffer',
             'vkCreateImage',
@@ -152,16 +166,11 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             'vkCmdSetViewport',
             'vkCmdSetScissor',
             'vkCmdSetLineWidth',
-            'vkCmdDraw',
             'vkCmdDrawIndirect',
             'vkCmdDrawIndexedIndirect',
             'vkCmdClearAttachments',
-            'vkCmdCopyImage',
             'vkCmdBindIndexBuffer',
-            'vkCmdBlitImage',
             'vkCmdCopyBuffer',
-            'vkCmdCopyBufferToImage',
-            'vkCmdCopyImageToBuffer',
             'vkCmdUpdateBuffer',
             'vkCmdFillBuffer',
             'vkCreateSwapchainKHR',
@@ -181,6 +190,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             'vkCreateAccelerationStructureNV',
             'vkCreateAccelerationStructureKHR',
             'vkGetAccelerationStructureHandleNV',
+            'vkGetPhysicalDeviceImageFormatProperties',
             'vkGetPhysicalDeviceImageFormatProperties2',
             'vkGetPhysicalDeviceImageFormatProperties2KHR',
             'vkCmdBuildAccelerationStructureNV',
@@ -218,8 +228,15 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             'vkCmdTraceRaysIndirectKHR',
             'vkCmdBuildAccelerationStructureIndirectKHR',
             'vkGetDeviceAccelerationStructureCompatibilityKHR',
-            'vkBuildAccelerationStructureKHR',
-            'vkCmdBuildAccelerationStructureKHR'
+            'vkCmdSetViewportWithCountEXT',
+            'vkCmdSetScissorWithCountEXT',
+            'vkCmdBindVertexBuffers2EXT',
+            'vkCmdCopyBuffer2KHR',
+            'vkCmdBuildAccelerationStructuresKHR',
+            'vkCmdBuildAccelerationStructuresIndirectKHR',
+            'vkBuildAccelerationStructuresKHR',
+            'vkGetAccelerationStructureBuildSizesKHR',
+            'vkCmdWriteAccelerationStructuresPropertiesNV'
             ]
 
         # Commands to ignore
@@ -235,7 +252,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             ]
 
         # Structure fields to ignore
-        self.structMemberBlacklist = { 'VkWriteDescriptorSet' : ['dstSet'] }
+        self.structMemberBlacklist = { 'VkWriteDescriptorSet' : ['dstSet'], 'VkAccelerationStructureGeometryKHR' :['geometry'] }
         # Validation conditions for some special case struct members that are conditionally validated
         self.structMemberValidationConditions = { 'VkPipelineColorBlendStateCreateInfo' : { 'logicOp' : '{}logicOpEnable == VK_TRUE' } }
         # Header version
@@ -250,6 +267,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
         self.validatedStructs = dict()                    # Map of structs type names to generated validation code for that struct type
         self.enumRanges = set()                           # Set of enum names
         self.enumValueLists = ''                          # String containing enumerated type map definitions
+        self.stype_version_dict = dict()                  # String containing structtype to version map data
         self.flags = set()                                # Map of flags typenames
         self.flagBits = dict()                            # Map of flag bits typename to list of values
         self.newFlags = set()                             # Map of flags typenames /defined in the current feature/
@@ -263,12 +281,14 @@ class ParameterValidationOutputGenerator(OutputGenerator):
         self.alias_dict = dict()                          # Dict of cmd|struct aliases
         self.header_file = False                          # Header file generation flag
         self.source_file = False                          # Source file generation flag
+        self.instance_extension_list = ''                 # List of instance extension name defines
+        self.device_extension_list = ''                   # List of device extension name defines
         self.returnedonly_structs = []
         # Named tuples to store struct and command data
         self.CommandParam = namedtuple('CommandParam', ['type', 'name', 'ispointer', 'isstaticarray', 'isbool', 'israngedenum',
                                                         'isconst', 'isoptional', 'iscount', 'noautovalidity',
                                                         'len', 'extstructs', 'condition', 'cdecl'])
-        self.CommandData = namedtuple('CommandData', ['name', 'params', 'cdecl', 'extension_type', 'result'])
+        self.CommandData = namedtuple('CommandData', ['name', 'params', 'cdecl', 'extension_type', 'result', 'promotion_info'])
         self.StructMemberData = namedtuple('StructMemberData', ['name', 'members'])
 
     #
@@ -344,6 +364,24 @@ class ParameterValidationOutputGenerator(OutputGenerator):
         if self.header_file:
             return
 
+        stype_map = ''
+        stype_version_dict = dict()
+        # Create contents of Structs->API version unordered map
+        root = self.registry.reg
+        for node in root.findall('feature'):
+            version_name = node.get('name')
+            version_name = version_name.replace('VK_', 'VK_API_')
+            for enum_item in node.iter('enum'):
+                if enum_item.get('extends') == "VkStructureType":
+                    struct_type_id = enum_item.get('name')
+                    self.stype_version_dict[struct_type_id] = version_name
+        for extensions in root.findall('extensions'):
+            for extension in extensions.findall('extension'):
+                for entry in extension.iterfind('require/enum[@extends="VkStructureType"]'):
+                    alias = entry.get('alias')
+                    if alias is not None and (entry.get('comment') is None or 'typo' not in entry.get('comment')):
+                        self.stype_version_dict[alias] = extension.get('name')
+
         # Build map of structure type names to VkStructureType enum values
         # Find all types of category "struct"
         for struct in self.registry.tree.iterfind('types/type[@category="struct"]'):
@@ -380,8 +418,38 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             self.newline()
             write(self.enumValueLists, file=self.outFile)
             self.newline()
+            self.newline()
 
-            pnext_handler  = 'bool StatelessValidation::ValidatePnextStructContents(const char *api_name, const ParameterName &parameter_name, const VkBaseOutStructure* header) const {\n'
+            api_func  = 'bool StatelessValidation::CheckPromotedApiAgainstVulkanVersion(VkInstance instance, const char *api_name, const uint32_t promoted_version) const {\n'
+            api_func += '    bool skip = false;\n'
+            api_func += '    if (api_version < promoted_version) {\n'
+            api_func += '        skip = LogError(instance,\n'
+            api_func += '                        kVUID_PVError_ApiVersionViolation, "Attemped to call %s() with an effective API version of %s"\n'
+            api_func += '                        "but this API was not promoted until version %s.", api_name, StringAPIVersion(api_version).c_str(),\n'
+            api_func += '                        StringAPIVersion(promoted_version).c_str());\n'
+            api_func += '    }\n'
+            api_func += '    return skip;\n'
+            api_func += '}\n\n'
+            api_func += 'bool StatelessValidation::CheckPromotedApiAgainstVulkanVersion(VkPhysicalDevice pdev, const char *api_name, const uint32_t promoted_version) const {\n'
+            api_func += '    bool skip = false;\n'
+            api_func += '    const auto &target_pdev = physical_device_properties_map.find(pdev);\n'
+            api_func += '    if (target_pdev != physical_device_properties_map.end()) {\n'
+            api_func += '        auto effective_api_version = std::min(target_pdev->second->apiVersion, api_version);\n'
+            api_func += '        if (effective_api_version < promoted_version) {\n'
+            api_func += '            skip = LogError(instance,\n'
+            api_func += '                            kVUID_PVError_ApiVersionViolation, "Attemped to call %s() with an effective API version of %s, "\n'
+            api_func += '                            "which is the minimum of version requested in pApplicationInfo (%s) and supported by this physical device (%s), "\n'
+            api_func += '                            "but this API was not promoted until version %s.", api_name, StringAPIVersion(api_version).c_str(),\n'
+            api_func += '                            StringAPIVersion(target_pdev->second->apiVersion).c_str(), StringAPIVersion(effective_api_version).c_str(),\n'
+            api_func += '                            StringAPIVersion(promoted_version).c_str());\n'
+            api_func += '        }\n'
+            api_func += '    }\n'
+            api_func += '    return skip;\n'
+            api_func += '}\n'
+            write(api_func, file=self.outFile)
+
+            pnext_handler  = 'bool StatelessValidation::ValidatePnextStructContents(const char *api_name, const ParameterName &parameter_name,\n'
+            pnext_handler += '                                                      const VkBaseOutStructure* header, const char *pnext_vuid) const {\n'
             pnext_handler += '    bool skip = false;\n'
             pnext_handler += '    switch(header->sType) {\n'
 
@@ -393,6 +461,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                 postProcSpec['ppi'] = '' if not item else '{postProcInsert}'
 
                 pnext_case = '\n'
+                pnext_check = ''
                 protect = ''
                 # Guard struct cases with feature ifdefs, if necessary
                 if item in self.struct_feature_protect.keys():
@@ -400,15 +469,60 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                     pnext_case += '#ifdef %s\n' % protect
                 pnext_case += '        // Validation code for %s structure members\n' % item
                 pnext_case += '        case %s: { // Covers VUID-%s-sType-sType\n' % (self.structTypes[item], item)
-                pnext_case += '            %s *structure = (%s *) header;\n' % (item, item)
+                # pNext version/extension-enabled checks
+                ver_info = ''
+                struct_type = self.structTypes[item]
+                if struct_type in self.stype_version_dict.keys():
+                    ver_info = self.stype_version_dict[struct_type]
+                else:
+                    struct_type[:-4]
+                    if struct_type[:-4] in self.stype_version_dict.values():
+                        ver_info = self.stype_version_dict[struct_type[:-4]]
+                    else:
+                        ver_info = None
+                if ver_info is not None:
+                    if 'VK_API_VERSION_' in ver_info:
+                        api_version = ver_info;
+                        pnext_check += '            if (api_version < %s) {\n' % ver_info
+                        pnext_check += '                skip |= LogError(\n'
+                        pnext_check += '                           instance, pnext_vuid,\n'
+                        pnext_check += '                           "%%s: Includes a pNext pointer (%%s) to a VkStructureType (%s) which was added in %s but the "\n' % (struct_type, ver_info)
+                        pnext_check += '                           "current effective API version is %s.",\n'
+                        pnext_check += '                           api_name, parameter_name.get_name().c_str(), StringAPIVersion(api_version).c_str());\n'
+                        pnext_check += '            }\n'
+                    else:
+                        # Dependent on enabled extension
+                        ext_name = ver_info
+                        ext_name_define = self.extension_names[ver_info]
+                        table_type = ''
+                        if ext_name_define in self.instance_extension_list:
+                            table_type = 'instance'
+                        elif ext_name_define in self.device_extension_list:
+                            table_type = 'device'
+                        else:
+                            print("Error in parameter_validation_generator.py CodeGen.")
+                        norm_ext_name = ext_name_define[:-15].lower()
+                        if table_type == 'device':
+                            pnext_check += '            if ((!SupportedByPdev(physical_device, %s)) && !%s_extensions.%s) {\n' % (ext_name_define, table_type, norm_ext_name.lower())
+                        else:
+                            pnext_check += '            if (!%s_extensions.%s) {\n' % (table_type, norm_ext_name.lower())
+                        pnext_check += '                skip |= LogError(\n'
+                        pnext_check += '                           instance, pnext_vuid,\n'
+                        pnext_check += '                           "%%s: Includes a pNext pointer (%%s) to a VkStructureType (%s), but its parent extension "\n' % struct_type
+                        pnext_check += '                           "%s has not been enabled.",\n' % ext_name
+                        pnext_check += '                           api_name, parameter_name.get_name().c_str());\n'
+                        pnext_check += '            }\n'
+                        pnext_check += '\n'
                 expr = self.expandStructCode(item, item, 'structure->', '', '            ', [], postProcSpec)
                 struct_validation_source = self.ScrubStructCode(expr)
-                pnext_case += '%s' % struct_validation_source
+                if struct_validation_source != '':
+                    pnext_case += '            %s *structure = (%s *) header;\n' % (item, item)
+                pnext_case += '%s%s' % (pnext_check, struct_validation_source)
                 pnext_case += '        } break;\n'
                 if protect:
                     pnext_case += '#endif // %s\n' % protect
                 # Skip functions containing no validation
-                if struct_validation_source:
+                if struct_validation_source or pnext_check != '':
                     pnext_handler += pnext_case;
                 else:
                     pnext_handler += '\n        // No Validation code for %s structure members  -- Covers VUID-%s-sType-sType\n' % (item, item)
@@ -452,9 +566,9 @@ class ParameterValidationOutputGenerator(OutputGenerator):
         # Get base list of extension dependencies for all items in this extension
         base_required_extensions = []
         if "VK_VERSION_1" not in self.featureName:
-            # Save Name Define to get correct enable name later
             nameElem = interface[0][1]
             name = nameElem.get('name')
+            # Save Name Define to get correct enable name later
             self.extension_names[self.featureName] = name
             # This extension is the first dependency for this command
             base_required_extensions.append(self.featureName)
@@ -477,6 +591,16 @@ class ParameterValidationOutputGenerator(OutputGenerator):
 
         # And note if this is an Instance or Device extension
         self.extension_type = interface.get('type')
+        if interface.tag == 'extension':
+            name_elem = interface[0][1]
+            name_definition = name_elem.get('name')
+            if 'EXTENSION_NAME' not in name_definition:
+                print("Error in vk.xml file -- extension name is not available")
+            if interface.get('type') == 'instance':
+                self.instance_extension_list += '%s, ' % name_definition
+            else:
+                self.device_extension_list += '%s, ' % name_definition
+
     #
     # Called at the end of each extension (feature)
     def endFeature(self):
@@ -546,6 +670,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             return
         # alias has already been recorded in genType, above
         OutputGenerator.genStruct(self, typeinfo, typeName, alias)
+
         conditions = self.structMemberValidationConditions[typeName] if typeName in self.structMemberValidationConditions else None
         members = typeinfo.elem.findall('.//member')
         if self.featureExtraProtect is not None:
@@ -670,7 +795,10 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                     self.declarations += [ '#ifdef %s' % self.featureExtraProtect ]
                 # Strip off 'vk' from API name
                 decl = '%s%s' % ('bool PreCallValidate', decls[0].split("VKAPI_CALL vk")[1])
-                decl = str(decl).replace(';', ' const;')
+                decl_terminator =  ' const override;'
+                if 'ValidationCache' in name:
+                    decl_terminator = ' const;'
+                decl = str(decl).replace(';', decl_terminator)
                 self.declarations += [ decl ]
                 if (self.featureExtraProtect is not None):
                     self.declarations += [ '#endif' ]
@@ -707,10 +835,14 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                                                         cdecl=cdecl))
                 # Save return value information, if any
                 result_type = ''
+                promotion_info = ''
                 resultinfo = cmdinfo.elem.find('proto/type')
                 if (resultinfo is not None and resultinfo.text != 'void'):
                     result_type = resultinfo.text
-                self.commands.append(self.CommandData(name=name, params=paramsInfo, cdecl=self.makeCDecls(cmdinfo.elem)[0], extension_type=self.extension_type, result=result_type))
+                if "VK_VERSION" in self.featureName and "VK_VERSION_1_0" != self.featureName:
+                    if ('VkInstance' == paramsInfo[0].type or 'VkPhysicalDevice' == paramsInfo[0].type):
+                        promotion_info = [paramsInfo[0].name, self.featureName]
+                self.commands.append(self.CommandData(name=name, params=paramsInfo, cdecl=self.makeCDecls(cmdinfo.elem)[0], extension_type=self.extension_type, result=result_type, promotion_info=promotion_info))
     #
     # Check if the parameter passed in is a pointer
     def paramIsPointer(self, param):
@@ -771,15 +903,22 @@ class ParameterValidationOutputGenerator(OutputGenerator):
     # Retrieve the value of the len tag
     def getLen(self, param):
         result = None
-        len = param.attrib.get('len')
+        # Default to altlen when available to avoid LaTeX markup
+        if 'altlen' in param.attrib:
+            len = param.attrib.get('altlen')
+        else:
+            len = param.attrib.get('len')
         if len and len != 'null-terminated':
-            # For string arrays, 'len' can look like 'count,null-terminated', indicating that we have a null terminated array of
-            # strings.  We strip the null-terminated from the 'len' field and only return the parameter specifying the string count
-            if 'null-terminated' in len:
-                result = len.split(',')[0]
-            else:
-                result = len
-            result = str(result).replace('::', '->')
+            # Only first level is supported for multidimensional arrays. Conveniently, this also strips the trailing
+            # 'null-terminated' from arrays of strings
+            len = len.split(',')[0]
+            # Convert scope notation to pointer access
+            result = str(len).replace('::', '->')
+        elif self.paramIsStaticArray(param):
+            # For static arrays get length from inside []
+            array_match = re.search(r'\[(\d+)\]', param.find('name').tail)
+            if array_match:
+                result = array_match.group(1)
         return result
     #
     # Retrieve the type and name for a parameter
@@ -800,27 +939,20 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                 return param
         return None
     #
-    # Get the length paramater record for the specified parameter name
-    def getLenParam(self, params, name):
-        lenParam = None
-        if name:
-            if '->' in name:
-                # The count is obtained by dereferencing a member of a struct parameter
-                lenParam = self.CommandParam(name=name, iscount=True, ispointer=False, isbool=False, israngedenum=False, isconst=False,
-                                             isstaticarray=None, isoptional=False, type=None, noautovalidity=False,
-                                             len=None, extstructs=None, condition=None, cdecl=None)
-            elif 'latexmath' in name:
-                # Filter list of params to those whose name appears in the LaTeX markup
-                len_candidates = [p for p in params if re.search(r'\b{}\b'.format(p.name), name)]
-                # 0 or 1 matches are expected, >1 would require a special case and/or explicit validation
-                if len(len_candidates) == 0:
-                    lenParam = None
-                elif len(len_candidates) == 1:
-                    lenParam = len_candidates[0]
-                else:
-                    raise Exception('Cannot determine length parameter for len attribute value {}'.format(name))
+    # Get the length paramater record for the specified length expression
+    def getLenParam(self, params, length):
+        # First check if any element of params matches length exactly
+        lenParam = self.getParamByName(params, length)
+        if not lenParam:
+            # Otherwise, look for any elements of params that appear within length
+            len_candidates = [p for p in params if re.search(r'\b{}\b'.format(p.name), length)]
+            # 0 or 1 matches are expected, >1 would require a special case and/or explicit validation
+            if len(len_candidates) == 0:
+                lenParam = None
+            elif len(len_candidates) == 1:
+                lenParam = len_candidates[0]
             else:
-                lenParam = self.getParamByName(params, name)
+                raise Exception('Cannot determine length parameter for len attribute value {}'.format(length))
         return lenParam
     #
     # Convert a vulkan.h command declaration into a parameter_validation.h definition
@@ -891,17 +1023,21 @@ class ParameterValidationOutputGenerator(OutputGenerator):
         param_vuid = self.GetVuid(vuid_name, "%s-parameter" % value.name)
 
         if lenValue:
-            count_required_vuid = self.GetVuid(vuid_name, "%s-arraylength" % lenValue.name)
+            count_required_vuid = self.GetVuid(vuid_name, "%s-arraylength" % value.len)
 
+            # This is an array of struct pointers
+            if value.ispointer == 2:
+                checkExpr.append('skip |= validate_struct_pointer_type_array("{}", {ppp}"{ldn}"{pps}, {ppp}"{dn}"{pps}, "{sv}", {pf}{ln}, {pf}{vn}, {sv}, {}, {}, {}, {}, {});\n'.format(
+                    funcPrintName, lenValueRequired, valueRequired, stype_vuid, param_vuid, count_required_vuid, ln=lenValue.name, ldn=lenPrintName, dn=valuePrintName, vn=value.name, sv=stype, pf=prefix, **postProcSpec))
             # This is an array with a pointer to a count value
-            if lenValue.ispointer:
+            elif lenValue.ispointer:
                 # When the length parameter is a pointer, there is an extra Boolean parameter in the function call to indicate if it is required
                 checkExpr.append('skip |= validate_struct_type_array("{}", {ppp}"{ldn}"{pps}, {ppp}"{dn}"{pps}, "{sv}", {pf}{ln}, {pf}{vn}, {sv}, {}, {}, {}, {}, {}, {});\n'.format(
-                    funcPrintName, lenPtrRequired, lenValueRequired, valueRequired, stype_vuid, param_vuid, count_required_vuid, ln=lenValue.name, ldn=lenPrintName, dn=valuePrintName, vn=value.name, sv=stype, pf=prefix, **postProcSpec))
+                    funcPrintName, lenPtrRequired, lenValueRequired, valueRequired, stype_vuid, param_vuid, count_required_vuid, ln=value.len, ldn=lenPrintName, dn=valuePrintName, vn=value.name, sv=stype, pf=prefix, **postProcSpec))
             # This is an array with an integer count value
             else:
                 checkExpr.append('skip |= validate_struct_type_array("{}", {ppp}"{ldn}"{pps}, {ppp}"{dn}"{pps}, "{sv}", {pf}{ln}, {pf}{vn}, {sv}, {}, {}, {}, {}, {});\n'.format(
-                    funcPrintName, lenValueRequired, valueRequired, stype_vuid, param_vuid, count_required_vuid, ln=lenValue.name, ldn=lenPrintName, dn=valuePrintName, vn=value.name, sv=stype, pf=prefix, **postProcSpec))
+                    funcPrintName, lenValueRequired, valueRequired, stype_vuid, param_vuid, count_required_vuid, ln=value.len, ldn=lenPrintName, dn=valuePrintName, vn=value.name, sv=stype, pf=prefix, **postProcSpec))
         # This is an individual struct
         else:
             checkExpr.append('skip |= validate_struct_type("{}", {ppp}"{}"{pps}, "{sv}", {}{vn}, {sv}, {}, {}, {});\n'.format(
@@ -918,7 +1054,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             else:
                 # This is an array with an integer count value
                 checkExpr.append('skip |= validate_handle_array("{}", {ppp}"{ldn}"{pps}, {ppp}"{dn}"{pps}, {pf}{ln}, {pf}{vn}, {}, {});\n'.format(
-                    funcPrintName, lenValueRequired, valueRequired, ln=lenValue.name, ldn=lenPrintName, dn=valuePrintName, vn=value.name, pf=prefix, **postProcSpec))
+                    funcPrintName, lenValueRequired, valueRequired, ln=value.len, ldn=lenPrintName, dn=valuePrintName, vn=value.name, pf=prefix, **postProcSpec))
         else:
             # This is assumed to be an output handle pointer
             raise('Unsupported parameter validation case: Output handles are not NULL checked')
@@ -932,7 +1068,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
             raise('Unsupported parameter validation case: array of reserved VkFlags')
         else:
             allFlags = 'All' + flagBitsName
-            checkExpr.append('skip |= validate_flags_array("{}", {ppp}"{}"{pps}, {ppp}"{}"{pps}, "{}", {}, {pf}{}, {pf}{}, {}, {});\n'.format(funcPrintName, lenPrintName, valuePrintName, flagBitsName, allFlags, lenValue.name, value.name, lenValueRequired, valueRequired, pf=prefix, **postProcSpec))
+            checkExpr.append('skip |= validate_flags_array("{}", {ppp}"{}"{pps}, {ppp}"{}"{pps}, "{}", {}, {pf}{}, {pf}{}, {}, {});\n'.format(funcPrintName, lenPrintName, valuePrintName, flagBitsName, allFlags, value.len, value.name, lenValueRequired, valueRequired, pf=prefix, **postProcSpec))
         return checkExpr
     #
     # Generate pNext check string
@@ -958,33 +1094,34 @@ class ParameterValidationOutputGenerator(OutputGenerator):
         checkExpr = []
         vuid_tag_name = struct_type_name if struct_type_name is not None else funcPrintName
         if lenValue:
-            count_required_vuid = self.GetVuid(vuid_tag_name, "%s-arraylength" % (lenValue.name))
+            length_deref = '->' in value.len
+            count_required_vuid = self.GetVuid(vuid_tag_name, "%s-arraylength" % (value.len))
             array_required_vuid = self.GetVuid(vuid_tag_name, "%s-parameter" % (value.name))
             # TODO: Remove workaround for missing optional tag in vk.xml
             if array_required_vuid == '"VUID-VkFramebufferCreateInfo-pAttachments-parameter"':
                 return []
             # This is an array with a pointer to a count value
-            if lenValue.ispointer:
+            if lenValue.ispointer and not length_deref:
                 # If count and array parameters are optional, there will be no validation
                 if valueRequired == 'true' or lenPtrRequired == 'true' or lenValueRequired == 'true':
                     # When the length parameter is a pointer, there is an extra Boolean parameter in the function call to indicate if it is required
                     checkExpr.append('skip |= validate_array("{}", {ppp}"{ldn}"{pps}, {ppp}"{dn}"{pps}, {pf}{ln}, &{pf}{vn}, {}, {}, {}, {}, {});\n'.format(
-                        funcPrintName, lenPtrRequired, lenValueRequired, valueRequired, count_required_vuid, array_required_vuid, ln=lenValue.name, ldn=lenPrintName, dn=valuePrintName, vn=value.name, pf=prefix, **postProcSpec))
+                        funcPrintName, lenPtrRequired, lenValueRequired, valueRequired, count_required_vuid, array_required_vuid, ln=value.len, ldn=lenPrintName, dn=valuePrintName, vn=value.name, pf=prefix, **postProcSpec))
             # This is an array with an integer count value
             else:
                 # If count and array parameters are optional, there will be no validation
                 if valueRequired == 'true' or lenValueRequired == 'true':
                     if value.type != 'char':
                         checkExpr.append('skip |= validate_array("{}", {ppp}"{ldn}"{pps}, {ppp}"{dn}"{pps}, {pf}{ln}, &{pf}{vn}, {}, {}, {}, {});\n'.format(
-                            funcPrintName, lenValueRequired, valueRequired, count_required_vuid, array_required_vuid, ln=lenValue.name, ldn=lenPrintName, dn=valuePrintName, vn=value.name, pf=prefix, **postProcSpec))
+                            funcPrintName, lenValueRequired, valueRequired, count_required_vuid, array_required_vuid, ln=value.len, ldn=lenPrintName, dn=valuePrintName, vn=value.name, pf=prefix, **postProcSpec))
                     else:
                         # Arrays of strings receive special processing
                         checkExpr.append('skip |= validate_string_array("{}", {ppp}"{ldn}"{pps}, {ppp}"{dn}"{pps}, {pf}{ln}, {pf}{vn}, {}, {}, {}, {});\n'.format(
-                            funcPrintName, lenValueRequired, valueRequired, count_required_vuid, array_required_vuid, ln=lenValue.name, ldn=lenPrintName, dn=valuePrintName, vn=value.name, pf=prefix, **postProcSpec))
+                            funcPrintName, lenValueRequired, valueRequired, count_required_vuid, array_required_vuid, ln=value.len, ldn=lenPrintName, dn=valuePrintName, vn=value.name, pf=prefix, **postProcSpec))
             if checkExpr:
-                if lenValue and ('->' in lenValue.name):
+                if lenValue and length_deref:
                     # Add checks to ensure the validation call does not dereference a NULL pointer to obtain the count
-                    checkExpr = self.genCheckedLengthCall(lenValue.name, checkExpr)
+                    checkExpr = self.genCheckedLengthCall(value.len, checkExpr)
         # This is an individual struct that is not allowed to be NULL
         elif not value.isoptional:
             # Function pointers need a reinterpret_cast to void*
@@ -1105,13 +1242,13 @@ class ParameterValidationOutputGenerator(OutputGenerator):
         indent = self.incIndent(None)
         if lenValue:
             # Need to process all elements in the array
-            indexName = lenValue.name.replace('Count', 'Index')
+            indexName = value.len.replace('Count', 'Index')
             expr[-1] += '\n'
             if lenValue.ispointer:
                 # If the length value is a pointer, de-reference it for the count.
-                expr.append(indent + 'for (uint32_t {iname} = 0; {iname} < *{}{}; ++{iname})\n'.format(prefix, lenValue.name, iname=indexName))
+                expr.append(indent + 'for (uint32_t {iname} = 0; {iname} < *{}{}; ++{iname})\n'.format(prefix, value.len, iname=indexName))
             else:
-                expr.append(indent + 'for (uint32_t {iname} = 0; {iname} < {}{}; ++{iname})\n'.format(prefix, lenValue.name, iname=indexName))
+                expr.append(indent + 'for (uint32_t {iname} = 0; {iname} < {}{}; ++{iname})\n'.format(prefix, value.len, iname=indexName))
             expr.append(indent + '{')
             indent = self.incIndent(indent)
             # Prefix for value name to display in error message
@@ -1165,7 +1302,7 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                     # The parameter is an array with an explicit count parameter
                     lenParam = self.getLenParam(values, value.len)
                     if lenParam:
-                        lenDisplayName = '{}{}'.format(displayNamePrefix, lenParam.name)
+                        lenDisplayName = value.len.replace(lenParam.name, displayNamePrefix + lenParam.name)
                         if lenParam.ispointer:
                             # Count parameters that are pointers are inout
                             if type(lenParam.isoptional) is list:
@@ -1179,6 +1316,11 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                         else:
                             if lenParam.isoptional:
                                 cvReq = 'false'
+                    else:
+                        # Do not generate length checks for constant sized arrays
+                        cpReq = 'false'
+                        cvReq = 'false'
+
                 #
                 # The parameter will not be processed when tagged as 'noautovalidity'
                 # For the pointer to struct case, the struct pointer will not be validated, but any
@@ -1198,10 +1340,10 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                     elif value.type in self.flags and value.isconst:
                         usedLines += self.makeFlagsArrayCheck(valuePrefix, value, lenParam, req, cvReq, funcName, lenDisplayName, valueDisplayName, postProcSpec)
                     elif value.isbool and value.isconst:
-                        usedLines.append('skip |= validate_bool32_array("{}", {ppp}"{}"{pps}, {ppp}"{}"{pps}, {pf}{}, {pf}{}, {}, {});\n'.format(funcName, lenDisplayName, valueDisplayName, lenParam.name, value.name, cvReq, req, pf=valuePrefix, **postProcSpec))
+                        usedLines.append('skip |= validate_bool32_array("{}", {ppp}"{}"{pps}, {ppp}"{}"{pps}, {pf}{}, {pf}{}, {}, {});\n'.format(funcName, lenDisplayName, valueDisplayName, value.len, value.name, cvReq, req, pf=valuePrefix, **postProcSpec))
                     elif value.israngedenum and value.isconst:
                         enum_value_list = 'All%sEnums' % value.type
-                        usedLines.append('skip |= validate_ranged_enum_array("{}", {ppp}"{}"{pps}, {ppp}"{}"{pps}, "{}", {}, {pf}{}, {pf}{}, {}, {});\n'.format(funcName, lenDisplayName, valueDisplayName, value.type, enum_value_list, lenParam.name, value.name, cvReq, req, pf=valuePrefix, **postProcSpec))
+                        usedLines.append('skip |= validate_ranged_enum_array("{}", {ppp}"{}"{pps}, {ppp}"{}"{pps}, "{}", {}, {pf}{}, {pf}{}, {}, {});\n'.format(funcName, lenDisplayName, valueDisplayName, value.type, enum_value_list, value.len, value.name, cvReq, req, pf=valuePrefix, **postProcSpec))
                     elif value.name == 'pNext':
                         usedLines += self.makeStructNextCheck(valuePrefix, value, funcName, valueDisplayName, postProcSpec, structTypeName)
                     else:
@@ -1316,6 +1458,10 @@ class ParameterValidationOutputGenerator(OutputGenerator):
                 func_sig = func_sig.split('VKAPI_CALL vk')[1]
                 cmdDef = 'bool StatelessValidation::PreCallValidate' + func_sig
                 cmdDef += '%sbool skip = false;\n' % indent
+                if isinstance(command.promotion_info, list):
+                    version_flag = command.promotion_info[1]
+                    version_id = version_flag.replace('VK_VERSION', 'VK_API_VERSION')
+                    cmdDef += '%s if (CheckPromotedApiAgainstVulkanVersion(%s, "%s", %s)) return true;\n' % (indent, command.promotion_info[0], command.name, version_id)
                 for line in lines:
                     if type(line) is list:
                         for sub in line:
