@@ -18,8 +18,14 @@
 
 void VkBestPracticesLayerTest::InitBestPracticesFramework() {
     // Enable all vendor-specific checks
-    SetEnvVar("VK_LAYER_ENABLES", "VALIDATION_CHECK_ENABLE_VENDOR_SPECIFIC_ALL");
-
+    VkLayerSettingValueDataEXT bp_setting_string_value{};
+    bp_setting_string_value.arrayString.pCharArray = "VALIDATION_CHECK_ENABLE_VENDOR_SPECIFIC_ALL";
+    bp_setting_string_value.arrayString.count = sizeof(bp_setting_string_value.arrayString.pCharArray);
+    VkLayerSettingValueEXT bp_vendor_all_setting_val = {"enables", VK_LAYER_SETTING_VALUE_TYPE_STRING_ARRAY_EXT,
+                                                        bp_setting_string_value};
+    VkLayerSettingsEXT bp_settings{static_cast<VkStructureType>(VK_STRUCTURE_TYPE_INSTANCE_LAYER_SETTINGS_EXT), nullptr, 1,
+                                   &bp_vendor_all_setting_val};
+    features_.pNext = &bp_settings;
     InitFramework(m_errorMonitor, &features_);
 }
 
@@ -107,6 +113,7 @@ TEST_F(VkBestPracticesLayerTest, UseDeprecatedInstanceExtensions) {
 
     // Create a 1.1 vulkan instance and request an extension promoted to core in 1.1
     m_errorMonitor->SetDesiredFailureMsg(kWarningBit, "UNASSIGNED-BestPractices-vkCreateInstance-deprecated-extension");
+    m_errorMonitor->SetDesiredFailureMsg(kWarningBit, "UNASSIGNED-BestPractices-vkCreateInstance-specialuse-extension");
     VkInstance dummy;
     auto features = features_;
     auto ici = GetInstanceCreateInfo();
@@ -182,6 +189,40 @@ TEST_F(VkBestPracticesLayerTest, UseDeprecatedDeviceExtensions) {
     m_errorMonitor->VerifyFound();
 }
 
+TEST_F(VkBestPracticesLayerTest, SpecialUseExtensions) {
+    TEST_DESCRIPTION("Create a device with a 'specialuse' extension.");
+
+    ASSERT_NO_FATAL_FAILURE(InitBestPracticesFramework());
+
+    if (DeviceExtensionSupported(gpu(), nullptr, VK_EXT_DEPTH_CLIP_ENABLE_EXTENSION_NAME)) {
+        m_device_extension_names.push_back(VK_EXT_DEPTH_CLIP_ENABLE_EXTENSION_NAME);
+    } else {
+        printf("%s %s Extension not supported, skipping test\n", kSkipPrefix, VK_EXT_DEPTH_CLIP_ENABLE_EXTENSION_NAME);
+        return;
+    }
+
+    VkDevice local_device;
+    VkDeviceCreateInfo dev_info = {};
+    VkDeviceQueueCreateInfo queue_info = {};
+    queue_info.sType = VK_STRUCTURE_TYPE_DEVICE_QUEUE_CREATE_INFO;
+    queue_info.pNext = NULL;
+    queue_info.queueFamilyIndex = 0;
+    queue_info.queueCount = 1;
+    queue_info.pQueuePriorities = nullptr;
+    dev_info.sType = VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO;
+    dev_info.pNext = nullptr;
+    dev_info.queueCreateInfoCount = 1;
+    dev_info.pQueueCreateInfos = &queue_info;
+    dev_info.enabledLayerCount = 0;
+    dev_info.ppEnabledLayerNames = NULL;
+    dev_info.enabledExtensionCount = m_device_extension_names.size();
+    dev_info.ppEnabledExtensionNames = m_device_extension_names.data();
+
+    m_errorMonitor->SetDesiredFailureMsg(kWarningBit, "UNASSIGNED-BestPractices-vkCreateDevice-specialuse-extension");
+    vk::CreateDevice(this->gpu(), &dev_info, NULL, &local_device);
+    m_errorMonitor->VerifyFound();
+}
+
 TEST_F(VkBestPracticesLayerTest, CmdClearAttachmentTest) {
     TEST_DESCRIPTION("Test for validating usage of vkCmdClearAttachments");
 
@@ -477,6 +518,12 @@ TEST_F(VkBestPracticesLayerTest, AttachmentShouldNotBeTransient) {
     InitBestPracticesFramework();
     InitState();
 
+    if (IsPlatform(kPixel2XL) || IsPlatform(kPixel3) || IsPlatform(kPixel3aXL) || IsPlatform(kShieldTV) || IsPlatform(kShieldTVb) ||
+        IsPlatform(kNexusPlayer)) {
+        printf("%s This test seems super-picky on Android platforms\n", kSkipPrefix);
+        return;
+    }
+
     m_errorMonitor->SetDesiredFailureMsg(kPerformanceWarningBit,
                                          "UNASSIGNED-BestPractices-vkCreateFramebuffer-attachment-should-not-be-transient");
 
@@ -616,454 +663,134 @@ TEST_F(VkBestPracticesLayerTest, ClearAttachmentsAfterLoad) {
     m_errorMonitor->VerifyFound();
 }
 
-// Tests for Arm-specific best practices
-
-TEST_F(VkArmBestPracticesLayerTest, TooManySamples) {
-    TEST_DESCRIPTION("Test for multisampled images with too many samples");
+TEST_F(VkBestPracticesLayerTest, TripleBufferingTest) {
+    TEST_DESCRIPTION("Test for usage of triple buffering");
 
+    AddSurfaceInstanceExtension();
     InitBestPracticesFramework();
+    AddSwapchainDeviceExtension();
     InitState();
-
     m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreateImage-too-large-sample-count");
+                                         "UNASSIGNED-BestPractices-vkCreateSwapchainKHR-suboptimal-swapchain-image-count");
+    InitSurface();
+    VkImageUsageFlags imageUsage = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
+    VkSurfaceTransformFlagBitsKHR preTransform = VK_SURFACE_TRANSFORM_IDENTITY_BIT_KHR;
+    VkSurfaceCapabilitiesKHR capabilities;
+    vk::GetPhysicalDeviceSurfaceCapabilitiesKHR(m_device->phy().handle(), m_surface, &capabilities);
 
-    VkImageCreateInfo image_info{};
-    image_info.sType = VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO;
-    image_info.extent = {1920, 1080, 1};
-    image_info.format = VK_FORMAT_R8G8B8A8_UNORM;
-    image_info.imageType = VK_IMAGE_TYPE_2D;
-    image_info.tiling = VK_IMAGE_TILING_OPTIMAL;
-    image_info.usage = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT | VK_IMAGE_USAGE_TRANSIENT_ATTACHMENT_BIT;
-    image_info.samples = VK_SAMPLE_COUNT_8_BIT;
-    image_info.arrayLayers = 1;
-    image_info.mipLevels = 1;
-
-    VkImage image = VK_NULL_HANDLE;
-    vk::CreateImage(m_device->device(), &image_info, nullptr, &image);
-
-    m_errorMonitor->VerifyFound();
-
-    if (image) {
-        vk::DestroyImage(m_device->device(), image, nullptr);
-    }
-}
-
-TEST_F(VkArmBestPracticesLayerTest, NonTransientMSImage) {
-    TEST_DESCRIPTION("Test for non-transient multisampled images");
-
-    InitBestPracticesFramework();
-    InitState();
-
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreateImage-non-transient-ms-image");
-
-    VkImageCreateInfo image_info{};
-    image_info.sType = VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO;
-    image_info.extent = {1920, 1080, 1};
-    image_info.format = VK_FORMAT_R8G8B8A8_UNORM;
-    image_info.imageType = VK_IMAGE_TYPE_2D;
-    image_info.tiling = VK_IMAGE_TILING_OPTIMAL;
-    image_info.usage = VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
-    image_info.samples = VK_SAMPLE_COUNT_4_BIT;
-    image_info.arrayLayers = 1;
-    image_info.mipLevels = 1;
-
-    VkImage image;
-    vk::CreateImage(m_device->device(), &image_info, nullptr, &image);
-
-    m_errorMonitor->VerifyFound();
-}
-
-TEST_F(VkArmBestPracticesLayerTest, SamplerCreation) {
-    TEST_DESCRIPTION("Test for various checks during sampler creation");
-
-    InitBestPracticesFramework();
-    InitState();
-
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreateSampler-different-wrapping-modes");
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreateSampler-lod-clamping");
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreateSampler-lod-bias");
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreateSampler-border-clamp-color");
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreateSampler-unnormalized-coordinates");
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreateSampler-anisotropy");
-
-    VkSamplerCreateInfo sampler_info{};
-    sampler_info.sType = VK_STRUCTURE_TYPE_SAMPLER_CREATE_INFO;
-    sampler_info.addressModeU = VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE;
-    sampler_info.addressModeV = VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_EDGE;
-    sampler_info.addressModeW = VK_SAMPLER_ADDRESS_MODE_CLAMP_TO_BORDER;
-    sampler_info.borderColor = VK_BORDER_COLOR_FLOAT_OPAQUE_BLACK;
-    sampler_info.minLod = 0.0f;
-    sampler_info.maxLod = 4.0f;
-    sampler_info.mipLodBias = 1.0f;
-    sampler_info.unnormalizedCoordinates = VK_TRUE;
-    sampler_info.anisotropyEnable = VK_TRUE;
-    sampler_info.maxAnisotropy = 4.0f;
-
-    VkSampler sampler = VK_NULL_HANDLE;
-    vk::CreateSampler(m_device->device(), &sampler_info, nullptr, &sampler);
-
-    m_errorMonitor->VerifyFound();
-
-    if (sampler) {
-        vk::DestroySampler(m_device->device(), sampler, nullptr);
-    }
-}
-
-TEST_F(VkArmBestPracticesLayerTest, MultisampledBlending) {
-    TEST_DESCRIPTION("Test for multisampled blending");
-
-    InitBestPracticesFramework();
-    InitState();
-
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCreatePipelines-multisampled-blending");
-
-    VkAttachmentDescription attachment{};
-    attachment.samples = VK_SAMPLE_COUNT_4_BIT;
-    attachment.loadOp = VK_ATTACHMENT_LOAD_OP_CLEAR;
-    attachment.storeOp = VK_ATTACHMENT_STORE_OP_DONT_CARE;
-    attachment.format = VK_FORMAT_R16G16B16A16_SFLOAT;
-
-    VkAttachmentReference color_ref{};
-    color_ref.attachment = 0;
-    color_ref.layout = VK_IMAGE_LAYOUT_COLOR_ATTACHMENT_OPTIMAL;
-
-    VkSubpassDescription subpass{};
-    subpass.colorAttachmentCount = 1;
-    subpass.pColorAttachments = &color_ref;
-    subpass.pipelineBindPoint = VK_PIPELINE_BIND_POINT_GRAPHICS;
-
-    VkRenderPassCreateInfo rp_info{};
-    rp_info.sType = VK_STRUCTURE_TYPE_RENDER_PASS_CREATE_INFO;
-    rp_info.attachmentCount = 1;
-    rp_info.pAttachments = &attachment;
-    rp_info.subpassCount = 1;
-    rp_info.pSubpasses = &subpass;
-
-    vk::CreateRenderPass(m_device->device(), &rp_info, nullptr, &m_renderPass);
-    m_renderPass_info = rp_info;
-
-    VkPipelineMultisampleStateCreateInfo pipe_ms_state_ci = {};
-    pipe_ms_state_ci.sType = VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO;
-    pipe_ms_state_ci.rasterizationSamples = VK_SAMPLE_COUNT_4_BIT;
-
-    VkPipelineColorBlendAttachmentState blend_att = {};
-    blend_att.blendEnable = VK_TRUE;
-    blend_att.colorWriteMask =
-        VK_COLOR_COMPONENT_R_BIT | VK_COLOR_COMPONENT_G_BIT | VK_COLOR_COMPONENT_B_BIT | VK_COLOR_COMPONENT_A_BIT;
-
-    VkPipelineColorBlendStateCreateInfo pipe_cb_state_ci = {};
-    pipe_cb_state_ci.sType = VK_STRUCTURE_TYPE_PIPELINE_COLOR_BLEND_STATE_CREATE_INFO;
-    pipe_cb_state_ci.attachmentCount = 1;
-    pipe_cb_state_ci.pAttachments = &blend_att;
-
-    CreatePipelineHelper pipe(*this);
-    pipe.InitInfo();
-    pipe.pipe_ms_state_ci_ = pipe_ms_state_ci;
-    pipe.cb_ci_ = pipe_cb_state_ci;
-    pipe.InitState();
-    pipe.CreateGraphicsPipeline();
-
-    m_errorMonitor->VerifyFound();
-}
-
-TEST_F(VkArmBestPracticesLayerTest, AttachmentNeedsReadback) {
-    TEST_DESCRIPTION("Test for attachments that need readback");
-
-    InitBestPracticesFramework();
-    InitState();
-
-    m_clear_via_load_op = false;  // Force LOAD_OP_LOAD
-    ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
-
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCmdBeginRenderPass-attachment-needs-readback");
-
-    m_commandBuffer->begin();
-    m_commandBuffer->BeginRenderPass(m_renderPassBeginInfo);
-
-    m_errorMonitor->VerifyFound();
-}
-
-TEST_F(VkArmBestPracticesLayerTest, ManySmallIndexedDrawcalls) {
-    InitBestPracticesFramework();
-    InitState();
-
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCmdDrawIndexed-many-small-indexed-drawcalls");
-
-    // This test may also trigger other warnings
-    m_errorMonitor->SetAllowedFailureMsg("UNASSIGNED-BestPractices-vkAllocateMemory-small-allocation");
-    m_errorMonitor->SetAllowedFailureMsg("UNASSIGNED-BestPractices-vkBindMemory-small-dedicated-allocation");
-
-    ASSERT_NO_FATAL_FAILURE(InitViewport());
-    ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
-
-    VkPipelineMultisampleStateCreateInfo pipe_ms_state_ci = {};
-    pipe_ms_state_ci.sType = VK_STRUCTURE_TYPE_PIPELINE_MULTISAMPLE_STATE_CREATE_INFO;
-    pipe_ms_state_ci.pNext = NULL;
-    pipe_ms_state_ci.rasterizationSamples = VK_SAMPLE_COUNT_1_BIT;
-    pipe_ms_state_ci.sampleShadingEnable = 0;
-    pipe_ms_state_ci.minSampleShading = 1.0;
-    pipe_ms_state_ci.pSampleMask = NULL;
-
-    CreatePipelineHelper pipe(*this);
-    pipe.InitInfo();
-    pipe.pipe_ms_state_ci_ = pipe_ms_state_ci;
-    pipe.InitState();
-    pipe.CreateGraphicsPipeline();
-
-    m_commandBuffer->begin();
-    m_commandBuffer->BeginRenderPass(m_renderPassBeginInfo);
-    vk::CmdBindPipeline(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.pipeline_);
-
-    for (int i = 0; i < 10; i++) {
-        m_commandBuffer->DrawIndexed(3, 1, 0, 0, 0);
+    uint32_t format_count;
+    vk::GetPhysicalDeviceSurfaceFormatsKHR(m_device->phy().handle(), m_surface, &format_count, nullptr);
+    vector<VkSurfaceFormatKHR> formats;
+    if (format_count != 0) {
+        formats.resize(format_count);
+        vk::GetPhysicalDeviceSurfaceFormatsKHR(m_device->phy().handle(), m_surface, &format_count, formats.data());
     }
 
+    uint32_t present_mode_count;
+    vk::GetPhysicalDeviceSurfacePresentModesKHR(m_device->phy().handle(), m_surface, &present_mode_count, nullptr);
+    vector<VkPresentModeKHR> present_modes;
+    if (present_mode_count != 0) {
+        present_modes.resize(present_mode_count);
+        vk::GetPhysicalDeviceSurfacePresentModesKHR(m_device->phy().handle(), m_surface, &present_mode_count, present_modes.data());
+    }
+
+    VkSwapchainCreateInfoKHR swapchain_create_info = {};
+    swapchain_create_info.sType = VK_STRUCTURE_TYPE_SWAPCHAIN_CREATE_INFO_KHR;
+    swapchain_create_info.pNext = 0;
+    swapchain_create_info.surface = m_surface;
+    swapchain_create_info.minImageCount = 2;
+    swapchain_create_info.imageFormat = formats[0].format;
+    swapchain_create_info.imageColorSpace = formats[0].colorSpace;
+    swapchain_create_info.imageExtent = {capabilities.minImageExtent.width, capabilities.minImageExtent.height};
+    swapchain_create_info.imageArrayLayers = 1;
+    swapchain_create_info.imageUsage = imageUsage;
+    swapchain_create_info.imageSharingMode = VK_SHARING_MODE_EXCLUSIVE;
+    swapchain_create_info.preTransform = preTransform;
+#ifdef VK_USE_PLATFORM_ANDROID_KHR
+    swapchain_create_info.compositeAlpha = VK_COMPOSITE_ALPHA_INHERIT_BIT_KHR;
+#else
+    swapchain_create_info.compositeAlpha = VK_COMPOSITE_ALPHA_OPAQUE_BIT_KHR;
+#endif
+    swapchain_create_info.presentMode = present_modes[0];
+    swapchain_create_info.clipped = VK_FALSE;
+    swapchain_create_info.oldSwapchain = 0;
+
+    VkResult err = vk::CreateSwapchainKHR(device(), &swapchain_create_info, nullptr, &m_swapchain);
     m_errorMonitor->VerifyFound();
 
-    m_commandBuffer->EndRenderPass();
-    m_commandBuffer->end();
-}
-
-TEST_F(VkArmBestPracticesLayerTest, SuboptimalDescriptorReuseTest) {
-    TEST_DESCRIPTION("Test for validation warnings of potentially suboptimal re-use of descriptor set allocations");
-
-    InitBestPracticesFramework();
-    InitState();
-    ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
-
-    VkDescriptorPoolSize ds_type_count = {};
-    ds_type_count.type = VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC;
-    ds_type_count.descriptorCount = 3;
-
-    VkDescriptorPoolCreateInfo ds_pool_ci = {};
-    ds_pool_ci.sType = VK_STRUCTURE_TYPE_DESCRIPTOR_POOL_CREATE_INFO;
-    ds_pool_ci.pNext = NULL;
-    ds_pool_ci.maxSets = 6;
-    ds_pool_ci.poolSizeCount = 1;
-    ds_pool_ci.flags = VK_DESCRIPTOR_POOL_CREATE_FREE_DESCRIPTOR_SET_BIT;
-    ds_pool_ci.pPoolSizes = &ds_type_count;
-
-    VkDescriptorPool ds_pool;
-    VkResult err = vk::CreateDescriptorPool(m_device->device(), &ds_pool_ci, NULL, &ds_pool);
-    ASSERT_VK_SUCCESS(err);
-
-    VkDescriptorSetLayoutBinding ds_binding = {};
-    ds_binding.binding = 0;
-    ds_binding.descriptorCount = 1;
-    ds_binding.descriptorType = VK_DESCRIPTOR_TYPE_UNIFORM_BUFFER_DYNAMIC;
-
-    VkDescriptorSetLayoutCreateInfo ds_layout_info = {};
-    ds_layout_info.sType = VK_STRUCTURE_TYPE_DESCRIPTOR_SET_LAYOUT_CREATE_INFO;
-    ds_layout_info.bindingCount = 1;
-    ds_layout_info.pBindings = &ds_binding;
-
-    VkDescriptorSetLayout ds_layout;
-    err = vk::CreateDescriptorSetLayout(m_device->device(), &ds_layout_info, nullptr, &ds_layout);
-    ASSERT_VK_SUCCESS(err);
-
-    auto ds_layouts = std::vector<VkDescriptorSetLayout>(ds_pool_ci.maxSets, ds_layout);
-
-    std::vector<VkDescriptorSet> descriptor_sets = {};
-    descriptor_sets.resize(ds_layouts.size());
-
-    // allocate N/2 descriptor sets
-    VkDescriptorSetAllocateInfo alloc_info = {};
-    alloc_info.sType = VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO;
-    alloc_info.descriptorPool = ds_pool;
-    alloc_info.descriptorSetCount = descriptor_sets.size() / 2;
-    alloc_info.pSetLayouts = ds_layouts.data();
-
-    err = vk::AllocateDescriptorSets(m_device->device(), &alloc_info, descriptor_sets.data());
-    ASSERT_VK_SUCCESS(err);
-
-    // free one descriptor set
-    VkDescriptorSet* ds = descriptor_sets.data();
-    err = vk::FreeDescriptorSets(m_device->device(), ds_pool, 1, ds);
-
-    // the previous allocate and free should not cause any warning
-    ASSERT_VK_SUCCESS(err);
+    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
+                                         "UNASSIGNED-BestPractices-vkCreateSwapchainKHR-suboptimal-swapchain-image-count");
+    swapchain_create_info.minImageCount = 3;
+    err = vk::CreateSwapchainKHR(device(), &swapchain_create_info, nullptr, &m_swapchain);
     m_errorMonitor->VerifyNotFound();
+    ASSERT_VK_SUCCESS(err)
+    DestroySwapchain();
+}
 
-    // allocate the previously freed descriptor set
-    alloc_info = {};
-    alloc_info.sType = VK_STRUCTURE_TYPE_DESCRIPTOR_SET_ALLOCATE_INFO;
-    alloc_info.descriptorPool = ds_pool;
-    alloc_info.descriptorSetCount = 1;
-    alloc_info.pSetLayouts = ds_layouts.data();
+TEST_F(VkBestPracticesLayerTest, ExpectedQueryDetails) {
+    TEST_DESCRIPTION("Check that GetPhysicalDeviceQueueFamilyProperties is working as expected");
 
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkAllocateDescriptorSets-suboptimal-reuse");
+    ASSERT_NO_FATAL_FAILURE(InitBestPracticesFramework());
+    const vk_testing::PhysicalDevice phys_device_obj(gpu_);
 
-    err = vk::AllocateDescriptorSets(m_device->device(), &alloc_info, ds);
+    std::vector<VkQueueFamilyProperties> queue_family_props;
 
-    // this should create a validation warning, in addition to the appropriate warning message
+    m_errorMonitor->ExpectSuccess(kErrorBit | kWarningBit);
+
+    // Ensure we can find a graphics queue family.
+    uint32_t queue_count = 0;
+    vk::GetPhysicalDeviceQueueFamilyProperties(phys_device_obj.handle(), &queue_count, nullptr);
+
+    queue_family_props.resize(queue_count);
+    vk::GetPhysicalDeviceQueueFamilyProperties(phys_device_obj.handle(), &queue_count, queue_family_props.data());
+
+    vk_testing::Device device(phys_device_obj.handle());
+    device.init();
+}
+
+TEST_F(VkBestPracticesLayerTest, MissingQueryDetails) {
+    TEST_DESCRIPTION("Check that GetPhysicalDeviceQueueFamilyProperties generates appropriate query warning");
+
+    ASSERT_NO_FATAL_FAILURE(InitBestPracticesFramework());
+    const vk_testing::PhysicalDevice phys_device_obj(gpu_);
+
+    std::vector<VkQueueFamilyProperties> queue_family_props(1);
+    uint32_t queue_count = static_cast<uint32_t>(queue_family_props.size());
+
+    m_errorMonitor->SetDesiredFailureMsg(kWarningBit, "UNASSIGNED-CoreValidation-DevLimit-MissingQueryCount");
+    vk::GetPhysicalDeviceQueueFamilyProperties(phys_device_obj.handle(), &queue_count, queue_family_props.data());
     m_errorMonitor->VerifyFound();
 
-    // allocate the remaining descriptor sets (N - (N/2))
-    alloc_info.descriptorSetCount = descriptor_sets.size() - (descriptor_sets.size() / 2);
-    err = vk::AllocateDescriptorSets(m_device->device(), &alloc_info, ds);
+    // Now get information correctly
+    m_errorMonitor->ExpectSuccess(kErrorBit | kWarningBit);
 
-    // this should create no validation warnings
-    m_errorMonitor->VerifyNotFound();
-}
-
-TEST_F(VkArmBestPracticesLayerTest, SparseIndexBufferTest) {
-    TEST_DESCRIPTION(
-        "Test for appropriate warnings to be thrown when recording an indexed draw call with sparse/non-sparse index buffers.");
-
-    InitBestPracticesFramework();
-    InitState();
-    ASSERT_NO_FATAL_FAILURE(InitViewport());
-    ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
-
-    if (IsPlatform(kMockICD) || DeviceSimulation()) {
-        printf("%s Test not supported by MockICD, skipping tests\n", kSkipPrefix);
-        return;
-    }
-
-    // create a non-sparse index buffer
-    std::vector<uint16_t> nonsparse_indices;
-    nonsparse_indices.resize(128);
-    for (unsigned i = 0; i < nonsparse_indices.size(); i++) {
-        nonsparse_indices[i] = i;
-    }
-
-    std::vector<uint16_t> sparse_indices = nonsparse_indices;
-    // The buffer (0, 1, 2, ..., n) is completely un-sparse. However, if n < 0xFFFF, by adding 0xFFFF at the end, we
-    // should trigger a warning due to loading all the indices in the range 0 to 0xFFFF, despite indices in the range
-    // (n+1) to (0xFFFF - 1) not being used.
-    sparse_indices[sparse_indices.size() - 1] = 0xFFFF;
-
-    VkConstantBufferObj nonsparse_ibo(m_device, nonsparse_indices.size() * sizeof(uint16_t), nonsparse_indices.data(),
-                                      VK_BUFFER_USAGE_INDEX_BUFFER_BIT);
-    VkConstantBufferObj sparse_ibo(m_device, sparse_indices.size() * sizeof(uint16_t), sparse_indices.data(),
-                                   VK_BUFFER_USAGE_INDEX_BUFFER_BIT);
-
-    CreatePipelineHelper pipe(*this);
-    pipe.InitInfo();
-    pipe.InitState();
-    pipe.ia_ci_.primitiveRestartEnable = VK_FALSE;
-    pipe.CreateGraphicsPipeline();
-
-    // pipeline with primitive restarts enabled
-    CreatePipelineHelper pr_pipe(*this);
-    pr_pipe.InitInfo();
-    pr_pipe.InitState();
-    pr_pipe.ia_ci_.primitiveRestartEnable = VK_TRUE;
-    pr_pipe.CreateGraphicsPipeline();
-
-    m_commandBuffer->begin();
-    m_commandBuffer->BeginRenderPass(m_renderPassBeginInfo);
-
-    auto test_pipelines = [&](VkConstantBufferObj& ibo, size_t index_count, bool expect_error) -> void {
-        vk::CmdBindPipeline(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.pipeline_);
-        m_commandBuffer->BindIndexBuffer(&ibo, static_cast<VkDeviceSize>(0), VK_INDEX_TYPE_UINT16);
-        m_errorMonitor->VerifyNotFound();
-
-        // the validation layer will only be able to analyse mapped memory, it's too expensive otherwise to do in the layer itself
-        ibo.memory().map();
-        if (expect_error)
-            m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                                 "UNASSIGNED-BestPractices-vkCmdDrawIndexed-sparse-index-buffer");
-        m_commandBuffer->DrawIndexed(index_count, 0, 0, 0, 0);
-        m_errorMonitor->VerifyFound();
-        ibo.memory().unmap();
-
-        vk::CmdBindPipeline(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pr_pipe.pipeline_);
-        m_commandBuffer->BindIndexBuffer(&ibo, static_cast<VkDeviceSize>(0), VK_INDEX_TYPE_UINT16);
-        m_errorMonitor->VerifyNotFound();
-
-        ibo.memory().map();
-        if (expect_error)
-            m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                                 "UNASSIGNED-BestPractices-vkCmdDrawIndexed-sparse-index-buffer");
-        m_commandBuffer->DrawIndexed(index_count, 0, 0, 0, 0);
-        m_errorMonitor->VerifyFound();
-        ibo.memory().unmap();
-    };
-
-    // our non-sparse indices should not trigger a warning for both pipelines in this case
-    test_pipelines(nonsparse_ibo, nonsparse_indices.size(), false);
-    // our sparse indices should trigger warnings for both pipelines in this case
-    test_pipelines(sparse_ibo, sparse_indices.size(), true);
-}
-
-TEST_F(VkArmBestPracticesLayerTest, PostTransformVertexCacheThrashingIndicesTest) {
-    TEST_DESCRIPTION(
-        "Test for appropriate warnings to be thrown when recording an indexed draw call where the indices thrash the "
-        "post-transform vertex cache.");
-
-    InitBestPracticesFramework();
-    InitState();
-    ASSERT_NO_FATAL_FAILURE(InitViewport());
-    ASSERT_NO_FATAL_FAILURE(InitRenderTarget());
-
-    if (IsPlatform(kMockICD) || DeviceSimulation()) {
-        printf("%s Test not supported by MockICD, skipping tests\n", kSkipPrefix);
-        return;
-    }
-
-    CreatePipelineHelper pipe(*this);
-    pipe.InitInfo();
-    pipe.InitState();
-    pipe.CreateGraphicsPipeline();
-
-    m_commandBuffer->begin();
-    m_commandBuffer->BeginRenderPass(m_renderPassBeginInfo);
-    vk::CmdBindPipeline(m_commandBuffer->handle(), VK_PIPELINE_BIND_POINT_GRAPHICS, pipe.pipeline_);
-
-    std::vector<uint16_t> worst_indices;
-    worst_indices.resize(128 * 16);
-    for (size_t i = 0; i < 16; i++) {
-        for (size_t j = 0; j < 128; j++) {
-            // worst case index buffer sequence for re-use
-            // (0, 1, 2, 3, ..., 127, 0, 1, 2, 3, ..., 127, 0, 1, 2, ...<x16>)
-            worst_indices[j + i * 128] = j;
+    vk_testing::QueueCreateInfoArray queue_info(phys_device_obj.queue_properties());
+    // Only request creation with queuefamilies that have at least one queue
+    std::vector<VkDeviceQueueCreateInfo> create_queue_infos;
+    auto qci = queue_info.data();
+    for (uint32_t j = 0; j < queue_info.size(); ++j) {
+        if (qci[j].queueCount) {
+            create_queue_infos.push_back(qci[j]);
         }
     }
-
-    std::vector<uint16_t> best_indices;
-    best_indices.resize(128 * 16);
-    for (size_t i = 0; i < 16; i++) {
-        for (size_t j = 0; j < 128; j++) {
-            // best case index buffer sequence for re-use
-            // (0, 0, 0, ...<x16>, 1, 1, 1, ...<x16>, 2, 2, 2, ...<x16> , ..., 127)
-            best_indices[i + j * 16] = j;
-        }
-    }
-
-    // make sure the worst-case indices throw a warning
-    VkConstantBufferObj worst_ibo(m_device, worst_indices.size() * sizeof(uint16_t), worst_indices.data(),
-                                  VK_BUFFER_USAGE_INDEX_BUFFER_BIT);
-    m_commandBuffer->BindIndexBuffer(&worst_ibo, static_cast<VkDeviceSize>(0), VK_INDEX_TYPE_UINT16);
     m_errorMonitor->VerifyNotFound();
 
-    // the validation layer will only be able to analyse mapped memory, it's too expensive otherwise to do in the layer itself
-    worst_ibo.memory().map();
-    m_errorMonitor->SetDesiredFailureMsg(VK_DEBUG_REPORT_PERFORMANCE_WARNING_BIT_EXT,
-                                         "UNASSIGNED-BestPractices-vkCmdDrawIndexed-post-transform-cache-thrashing");
-    m_commandBuffer->DrawIndexed(worst_indices.size(), 0, 0, 0, 0);
+    VkPhysicalDeviceFeatures all_features;
+    VkDeviceCreateInfo device_ci = {};
+    device_ci.sType = VK_STRUCTURE_TYPE_DEVICE_CREATE_INFO;
+    device_ci.pNext = nullptr;
+    device_ci.queueCreateInfoCount = create_queue_infos.size();
+    device_ci.pQueueCreateInfos = create_queue_infos.data();
+    device_ci.enabledLayerCount = 0;
+    device_ci.ppEnabledLayerNames = NULL;
+    device_ci.enabledExtensionCount = 0;
+    device_ci.ppEnabledExtensionNames = nullptr;
+    device_ci.pEnabledFeatures = &all_features;
+
+    // vkGetPhysicalDeviceFeatures has not been called, so this should produce a warning
+    m_errorMonitor->SetDesiredFailureMsg(kWarningBit,
+                                         "UNASSIGNED-BestPractices-vkCreateDevice-physical-device-features-not-retrieved");
+    VkDevice device;
+    vk::CreateDevice(phys_device_obj.handle(), &device_ci, nullptr, &device);
     m_errorMonitor->VerifyFound();
-    worst_ibo.memory().unmap();
-
-    // make sure that the best-case indices don't throw a warning
-    VkConstantBufferObj best_ibo(m_device, best_indices.size() * sizeof(uint16_t), best_indices.data(),
-                                 VK_BUFFER_USAGE_INDEX_BUFFER_BIT);
-    m_commandBuffer->BindIndexBuffer(&best_ibo, static_cast<VkDeviceSize>(0), VK_INDEX_TYPE_UINT16);
-    m_errorMonitor->VerifyNotFound();
-
-    best_ibo.memory().map();
-    m_commandBuffer->DrawIndexed(best_indices.size(), 0, 0, 0, 0);
-    m_errorMonitor->VerifyNotFound();
-    best_ibo.memory().unmap();
 }
