@@ -207,7 +207,9 @@ typedef enum {
     kPixelC,
     kNexusPlayer,
     kShieldTV,
+    kShieldTVb,
     kPixel3aXL,
+    kPixel2XL,
     kMockICD,
 } PlatformType;
 
@@ -217,7 +219,9 @@ const std::unordered_map<PlatformType, std::string, std::hash<int>> vk_gpu_table
     {kPixelC, "NVIDIA Tegra X1"},
     {kNexusPlayer, "PowerVR Rogue G6430"},
     {kShieldTV, "NVIDIA Tegra X1 (nvgpu)"},
+    {kShieldTVb, "NVIDIA Tegra X1 (rev B) (nvgpu)"},
     {kPixel3aXL, "Adreno (TM) 615"},
+    {kPixel2XL, "Adreno (TM) 540"},
     {kMockICD, "Vulkan Mock Device"},
 };
 
@@ -337,7 +341,7 @@ class VkPipelineObj;
 typedef vk_testing::Fence VkFenceObj;
 typedef vk_testing::Buffer VkBufferObj;
 typedef vk_testing::AccelerationStructure VkAccelerationStructureObj;
-
+typedef vk_testing::AccelerationStructureKHR VkAccelerationStructurekhrObj;
 class VkCommandPoolObj : public vk_testing::CommandPool {
   public:
     VkCommandPoolObj(VkDeviceObj *device, uint32_t queue_family_index, VkCommandPoolCreateFlags flags = 0);
@@ -400,6 +404,7 @@ class VkConstantBufferObj : public VkBufferObj {
 class VkRenderpassObj {
   public:
     VkRenderpassObj(VkDeviceObj *device, VkFormat format = VK_FORMAT_B8G8R8A8_UNORM);
+    VkRenderpassObj(VkDeviceObj *device, VkFormat format, bool depthStencil);
     ~VkRenderpassObj() NOEXCEPT;
     VkRenderPass handle() { return m_renderpass; }
 
@@ -414,15 +419,23 @@ class VkImageObj : public vk_testing::Image {
     bool IsCompatible(VkImageUsageFlags usages, VkFormatFeatureFlags features);
 
   public:
+    static VkImageCreateInfo ImageCreateInfo2D(uint32_t const width, uint32_t const height, uint32_t const mipLevels,
+                                               uint32_t const layers, VkFormat const format, VkFlags const usage,
+                                               VkImageTiling const requested_tiling = VK_IMAGE_TILING_LINEAR,
+                                               const std::vector<uint32_t> *queue_families = nullptr);
     void Init(uint32_t const width, uint32_t const height, uint32_t const mipLevels, VkFormat const format, VkFlags const usage,
               VkImageTiling const tiling = VK_IMAGE_TILING_LINEAR, VkMemoryPropertyFlags const reqs = 0,
               const std::vector<uint32_t> *queue_families = nullptr, bool memory = true);
+    void Init(const VkImageCreateInfo &create_info, VkMemoryPropertyFlags const reqs = 0, bool memory = true);
+
     void init(const VkImageCreateInfo *create_info);
 
     void InitNoLayout(uint32_t const width, uint32_t const height, uint32_t const mipLevels, VkFormat const format,
                       VkFlags const usage, VkImageTiling tiling = VK_IMAGE_TILING_LINEAR, VkMemoryPropertyFlags reqs = 0,
                       const std::vector<uint32_t> *queue_families = nullptr, bool memory = true);
 
+    void InitNoLayout(const VkImageCreateInfo &create_info, VkMemoryPropertyFlags reqs = 0, bool memory = true);
+
     //    void clear( CommandBuffer*, uint32_t[4] );
 
     void Layout(VkImageLayout const layout) { m_descriptorImageInfo.imageLayout = layout; }
@@ -447,18 +460,20 @@ class VkImageObj : public vk_testing::Image {
 
     VkImage image() const { return handle(); }
 
-    VkImageView targetView(VkFormat format, VkImageAspectFlags aspect = VK_IMAGE_ASPECT_COLOR_BIT) {
+    VkImageView targetView(VkFormat format, VkImageAspectFlags aspect = VK_IMAGE_ASPECT_COLOR_BIT, uint32_t baseMipLevel = 0,
+                           uint32_t levelCount = VK_REMAINING_MIP_LEVELS, uint32_t baseArrayLayer = 0,
+                           uint32_t layerCount = VK_REMAINING_ARRAY_LAYERS, VkImageViewType type = VK_IMAGE_VIEW_TYPE_2D) {
         if (!m_targetView.initialized()) {
             VkImageViewCreateInfo createView = {};
             createView.sType = VK_STRUCTURE_TYPE_IMAGE_VIEW_CREATE_INFO;
             createView.image = handle();
-            createView.viewType = VK_IMAGE_VIEW_TYPE_2D;
+            createView.viewType = type;
             createView.format = format;
             createView.components.r = VK_COMPONENT_SWIZZLE_R;
             createView.components.g = VK_COMPONENT_SWIZZLE_G;
             createView.components.b = VK_COMPONENT_SWIZZLE_B;
             createView.components.a = VK_COMPONENT_SWIZZLE_A;
-            createView.subresourceRange = {aspect, 0, 1, 0, 1};
+            createView.subresourceRange = {aspect, baseMipLevel, levelCount, baseArrayLayer, layerCount};
             createView.flags = 0;
             m_targetView.init(*m_device, createView);
         }
@@ -478,6 +493,8 @@ class VkImageObj : public vk_testing::Image {
 
     vk_testing::ImageView m_targetView;
     VkDescriptorImageInfo m_descriptorImageInfo;
+    uint32_t m_mipLevels;
+    uint32_t m_arrayLayers;
 };
 
 class VkTextureObj : public VkImageObj {
