@@ -408,6 +408,11 @@ class Buffer : public internal::NonDispHandle<VkBuffer> {
             init_no_mem(dev,
                         create_info(size, VK_BUFFER_USAGE_TRANSFER_SRC_BIT | VK_BUFFER_USAGE_TRANSFER_DST_BIT, queue_families));
     }
+    void init_as_storage(const Device &dev, VkDeviceSize size, VkMemoryPropertyFlags &reqs,
+        const std::vector<uint32_t> *queue_families = nullptr) {
+        init(dev, create_info(size, VK_BUFFER_USAGE_STORAGE_BUFFER_BIT, queue_families), reqs);
+    }
+
     void init_no_mem(const Device &dev, const VkBufferCreateInfo &info);
 
     // get the internal memory
@@ -564,7 +569,6 @@ class AccelerationStructure : public internal::NonDispHandle<VkAccelerationStruc
 
     // vkCreateAccelerationStructureNV
     void init(const Device &dev, const VkAccelerationStructureCreateInfoNV &info, bool init_memory = true);
-
     // vkGetAccelerationStructureMemoryRequirementsNV()
     VkMemoryRequirements2 memory_requirements() const;
     VkMemoryRequirements2 build_scratch_memory_requirements() const;
@@ -583,6 +587,29 @@ class AccelerationStructure : public internal::NonDispHandle<VkAccelerationStruc
     uint64_t opaque_handle_;
 };
 
+class AccelerationStructureKHR : public internal::NonDispHandle<VkAccelerationStructureKHR> {
+  public:
+    explicit AccelerationStructureKHR(const Device &dev, const VkAccelerationStructureCreateInfoKHR &info,
+                                      bool init_memory = true) {
+        init(dev, info, init_memory);
+    }
+    ~AccelerationStructureKHR();
+    // vkCreateAccelerationStructureNV
+    void init(const Device &dev, const VkAccelerationStructureCreateInfoKHR &info, bool init_memory = true);
+    uint64_t opaque_handle() const { return opaque_handle_; }
+
+    const VkAccelerationStructureCreateInfoKHR &info() const { return info_; }
+
+    const VkDevice &dev() const { return device(); }
+
+    void create_scratch_buffer(const Device &dev, Buffer *buffer, VkBufferCreateInfo *pCreateInfo = NULL);
+
+  private:
+    VkAccelerationStructureCreateInfoKHR info_;
+    DeviceMemory memory_;
+    uint64_t opaque_handle_;
+};
+
 class ShaderModule : public internal::NonDispHandle<VkShaderModule> {
   public:
     ~ShaderModule() NOEXCEPT;
